use crate::base::{entity::BaseEntity, i_service::IBaseService};
use anyhow::Result;
use axum::{extract::{Path, State}, http::StatusCode, response::IntoResponse, routing::get, Json, Router};
use diesel::Identifiable;
use serde::Serialize;
use std::{marker::PhantomData, ops::Deref, sync::Arc};

pub struct BaseController<T, S>
where
    T: BaseEntity + Identifiable<Id = i32> + Serialize + Send + Sync + 'static,
    S: IBaseService<T> + Send + Sync + 'static,
{
    service: S,
    _marker: PhantomData<T>,
}

impl<T, S> BaseController<T, S>
where
    T: BaseEntity + Identifiable<Id = i32> + Serialize + Send + Sync + 'static,
    S: IBaseService<T> + Send + Sync + 'static,
{
    pub fn new(service: S) -> Self {
        Self {
            service,
            _marker: PhantomData,
        }
    }

    pub async fn get_all(&self) -> Result<impl IntoResponse, String> {
        let entities = self
            .service
            .get_all()
            .await
            .map_err(|err| err.to_string())?;
        Ok((StatusCode::OK, Json(entities)))
    }

    pub async fn get_by_id(&self, id: i32) -> Result<impl IntoResponse, String> {
        let entity = self
            .service
            .get_by_id(id)
            .await
            .map_err(|err| err.to_string())?;

        match entity {
            Some(entity) => Ok((StatusCode::OK, Json(entity))),
            None => Err(format!("Entity with id {} not found.", id))
        }
    }

    pub fn default_routes<C>(controller: Arc<C>, base_path: &str) -> Router
    where
        C: Deref<Target = Self> + Send + Sync + 'static,
    {
        Router::new()
            .route(
                &format!("{}", base_path),
                get(|State(controller): State<Arc<C>>| async move {
                    controller
                        .get_all()
                        .await
                        .map_err(|err| (StatusCode::INTERNAL_SERVER_ERROR, Json(err)))
                }),
            )

            .route(
                // Add route for getting by ID (e.g., /:id)
                &format!("{}/{{id}}", base_path),
                get(|State(controller): State<Arc<C>>, Path(id): Path<i32>| async move {
                    controller
                        .get_by_id(id)
                        .await
                        .map_err(|err| (StatusCode::NOT_FOUND, Json(err)))
                }),
            )
            .with_state(controller)
    }
}
